from setuptools import setup

setup(name='commondist',
      version='0.2',
      description='Common distributions',
      packages=['commondist'],
      author = 'Jason Ho',
      author_email = 'jasonho0810@gmail.com',
      zip_safe=False)
